@@ -1,9 +1,11 @@
 import datetime
 import msgpack
 import os
+import base64
 import socket
 import sys
 import threading
+import Queue
 
 from bwtypes import *
 
@@ -58,7 +60,10 @@ class Client(object):
                         result = BosswaveResult(from_, uri, frame.kv_pairs,
                                                 frame.routing_objects,
                                                 frame.payload_objects)
-                    message_handler(result)
+                    # Place message handler and result in message queue.
+                    # This allows callbacks to do bosswave actions (i.e. publish/subscribe)
+                    # because they now take place from another thread.
+                    self.msgq.put((message_handler, result))
                 elif list_result_handler is not None:
                     child = frame.getFirstValue("child")
                     if child is not None:
@@ -67,6 +72,13 @@ class Client(object):
                         list_result_handler(None)
 
 
+    # thread for executing callbacks
+    def _msgq_handler(self):
+        while True:
+            handler, item = self.msgq.get()
+            handler(item)
+            self.msgq.task_done()
+
     def __init__(self, host_name=None, port=None):
         default_host = "localhost"
         default_port = 28589
@@ -90,6 +102,12 @@ class Client(object):
         self.port = port
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
 
+        # setup message queue for handling callbacks
+        self.msgq = Queue.Queue()
+        msgq_worker = threading.Thread(target=self._msgq_handler)
+        msgq_worker.daemon = True
+        msgq_worker.start()
+
         self.response_handlers = {}
         self.response_handlers_lock = threading.Lock()
         self.result_handlers = {}
@@ -931,4 +949,63 @@ class Client(object):
             del self.synchronous_cond_vars[seq_num]
 
         if result.status != "okay":
-            raise RuntimeError("View publish failed: " + response.reason)
+            raise RuntimeError("View publish failed: " + result.reason)
+
+    def resolveAlias(self, alias):
+        seq_num = Frame.generateSequenceNumber()
+        frame = Frame("resa", seq_num)
+        frame.addKVPair("longkey", alias)
+
+        def responseHandler(response):
+            with self.synchronous_results_lock:
+                self.synchronous_results[seq_num] = response
+                self.synchronous_cond_vars[seq_num].notify()
+
+        with self.response_handlers_lock:
+            self.response_handlers[seq_num] = responseHandler
+        with self.synchronous_results_lock:
+            self.synchronous_cond_vars[seq_num] = \
+                    threading.Condition(self.synchronous_results_lock)
+        frame.writeToSocket(self.socket)
+
+        with self.synchronous_results_lock:
+            while not seq_num in self.synchronous_results:
+                self.synchronous_cond_vars[seq_num].wait()
+            result = self.synchronous_results.pop(seq_num)
+            del self.synchronous_cond_vars[seq_num]
+
+        if result.status != "okay":
+            raise RuntimeError("Resolve failed: " + result.reason)
+        val = result.getFirstValue("value")
+        if val is not None:
+            return base64.urlsafe_b64encode(str(val))
+        else:
+            return None
+
+    def unresolveAlias(self, b64_blob):
+        blob = base64.urlsafe_b64decode(b64_blob)
+        seq_num = Frame.generateSequenceNumber()
+        frame = Frame("resa", seq_num)
+        frame.addKVPair("unresolve", blob)
+
+        def responseHandler(response):
+            with self.synchronous_results_lock:
+                self.synchronous_results[seq_num] = response
+                self.synchronous_cond_vars[seq_num].notify()
+
+        with self.response_handlers_lock:
+            self.response_handlers[seq_num] = responseHandler
+        with self.synchronous_results_lock:
+            self.synchronous_cond_vars[seq_num] = \
+                    threading.Condition(self.synchronous_results_lock)
+        frame.writeToSocket(self.socket)
+
+        with self.synchronous_results_lock:
+            while not seq_num in self.synchronous_results:
+                self.synchronous_cond_vars[seq_num].wait()
+            result = self.synchronous_results.pop(seq_num)
+            del self.synchronous_cond_vars[seq_num]
+
+        if result.status != "okay":
+            raise RuntimeError("Unresolve failed: " + result.reason)
+        return result.getFirstValue("value")
